package org.sevensource.commons.configuration.jdbc.impl.spring;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.lang.StringUtils;
import org.sevensource.commons.configuration.jdbc.ConfigurationException;
import org.sevensource.commons.configuration.jdbc.database.ExtendedCompositeConfiguration;
import org.sevensource.commons.configuration.jdbc.database.ExtendedDatabaseConfigurationFactory;
import org.sevensource.commons.configuration.jdbc.impl.DefaultConfiguration;
import org.sevensource.commons.configuration.jdbc.impl.DefaultConstants;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;


@Configuration
public abstract class AbstractDefaultConfigConfiguration<T extends DefaultConfiguration> {
	
	public static final String CONFIGURATION_BEAN_NAME = "configuration";
	public final static String CONFIGURATIONS_DATASOURCE_NAME = "configurationsDataSource";
	
	
	/**
	 * Creates a new spring bean "configuration", which is a fully configured
	 * {@link Configuration} object
	 * 
	 * @param env
	 * @return
	 */
	@Bean(name=CONFIGURATION_BEAN_NAME)
	@Inject
	public T configuration(Environment env) {
		ExtendedCompositeConfiguration compositeDatabaseConfiguration = compositeDatabaseConfiguration(env);
		T configuration = getConfigurationInstance(env, compositeDatabaseConfiguration);
		configuration.setServerName(resolveServerName(env));
		configuration.setApplicationName(resolveApplicationName(env));
		
		return configuration;
	}
	
	/**
	 * builds the composite configuration (ie. multiple overlaying database configurations)
	 * @see CompositeConfiguration
	 * @see ExtendedCompositeConfiguration
	 * @param env
	 * @return
	 */
	protected ExtendedCompositeConfiguration compositeDatabaseConfiguration(Environment env) {
		DataSource dataSource = getDataSource();
		
		ExtendedDatabaseConfigurationFactory factory = databaseConfigurationFactory(dataSource, env);
		return factory.buildConfiguration();
	}
	
	
	protected String resolveServerName(Environment env) {
		final String name = env.getProperty(DefaultConstants.SERVER_NAME);
		if(StringUtils.isBlank(name)) {
			final String msg = String.format("Please specify a servername (Parameter [%s])", DefaultConstants.SERVER_NAME);
			throw new ConfigurationException(msg);
		}
		return name;
	}
	
	protected String resolveApplicationName(Environment env) {		
		final String name = env.getProperty(DefaultConstants.APPLICATION_NAME);
		if(StringUtils.isBlank(name)) {
			final String msg = String.format("Please specify a applicationname (Parameter [%s])", DefaultConstants.APPLICATION_NAME);
			throw new ConfigurationException(msg);
		}
		return name;
	}
	
	/**
	 * provides the {@link DataSource} to use for looking up configurations
	 * @return
	 */
	protected abstract DataSource getDataSource();
	
	/**
	 * create a new {@link Configuration} instance (or a subclass of it)
	 * needs to setup the {@link Configuration} object with its most basic values
	 * @param env
	 * @param compositeDatabaseConfiguration
	 * @return
	 */
	protected abstract T getConfigurationInstance(Environment env, ExtendedCompositeConfiguration compositeDatabaseConfiguration);
	
	/**
	 * 
	 * @param dataSource
	 * @param env
	 * @return
	 */
	protected abstract ExtendedDatabaseConfigurationFactory databaseConfigurationFactory(DataSource dataSource, Environment env);
}
